@@ -44,6 +44,7 @@
 @class SOGoSentFolder;
 @class SOGoTrashFolder;
 @class SOGoJunkFolder;
+@class SOGoTemplatesFolder;
 
 typedef enum {
   undefined = -1,
@@ -63,6 +64,7 @@ typedef enum {
   SOGoSentFolder *sentFolder;
   SOGoTrashFolder *trashFolder;
   SOGoJunkFolder *junkFolder;
+  SOGoTemplatesFolder *templatesFolder;
   SOGoIMAPAclStyle imapAclStyle;
   NSMutableArray *identities;
   NSString *otherUsersFolderName;
@@ -117,12 +119,14 @@ typedef enum {
 - (NSString *) otherUsersFolderNameInContext: (id)_ctx;
 - (NSString *) sharedFoldersNameInContext: (id)_ctx;
 - (NSString *) junkFolderNameInContext: (id)_ctx;
+- (NSString *) templatesFolderNameInContext: (id)_ctx;
 
 - (SOGoMailFolder *) inboxFolderInContext: (id)_ctx;
 - (SOGoDraftsFolder *) draftsFolderInContext: (id)_ctx;
 - (SOGoSentFolder *) sentFolderInContext: (id)_ctx;
 - (SOGoTrashFolder *) trashFolderInContext: (id)_ctx;
 - (SOGoJunkFolder *) junkFolderInContext: (id)_ctx;
+- (SOGoTemplatesFolder *) templatesFolderInContext: (id)_ctx;
 
 /* namespaces */
 
