@@ -30,7 +30,12 @@
 #import <GDLContentStore/GCSChannelManager.h>
 #import <GDLContentStore/NSURL+GCS.h>
 #import <GDLContentStore/EOQualifier+GCS.h>
+#import <GDLAccess/EOAdaptor.h>
 #import <GDLAccess/EOAdaptorChannel.h>
+#import <GDLAccess/EOAdaptorContext.h>
+#import <GDLAccess/EOAttribute.h>
+#import <GDLAccess/EOEntity.h>
+#import <GDLAccess/EOSQLQualifier.h>
 
 #import <SOGo/SOGoSystemDefaults.h>
 
@@ -39,6 +44,7 @@
 #import "NSString+Crypto.h"
 
 #import "SQLSource.h"
+#import "SOGoPasswordPolicy.h"
 
 /**
  * The view MUST contain the following columns:
@@ -93,6 +99,7 @@
       // "mail" expands to all entries of MailFieldNames
       _searchFields = [NSArray arrayWithObjects: @"c_cn", @"mail", nil];
       [_searchFields retain];
+      _userPasswordPolicy = nil;
       _userPasswordAlgorithm = nil;
       _keyPath = nil;
       _viewURL = nil;
@@ -114,6 +121,7 @@
   [_loginFields release];
   [_mailFields release];
   [_searchFields release];
+  [_userPasswordPolicy release];
   [_userPasswordAlgorithm release];
   [_keyPath release];
   [_viewURL release];
@@ -138,6 +146,7 @@
   ASSIGN(_authenticationFilter, [udSource objectForKey: @"authenticationFilter"]);
   ASSIGN(_loginFields, [udSource objectForKey: @"LoginFieldNames"]);
   ASSIGN(_mailFields, [udSource objectForKey: @"MailFieldNames"]);
+  ASSIGN(_userPasswordPolicy, [SOGoPasswordPolicy createPasswordPolicyRegex: [udSource objectForKey: @"userPasswordPolicy"]]);
   ASSIGN(_userPasswordAlgorithm, [udSource objectForKey: @"userPasswordAlgorithm"]);
   ASSIGN(_keyPath, [udSource objectForKey: @"keyPath"]);
   ASSIGN(_imapLoginField, [udSource objectForKey: @"IMAPLoginFieldName"]);
@@ -187,6 +196,11 @@
   return _searchFields;
 }
 
+- (NSArray *) userPasswordPolicy
+{
+  return _userPasswordPolicy;
+}
+
 - (BOOL) _isPassword: (NSString *) plainPassword
              equalTo: (NSString *) encryptedPassword
 {
@@ -290,7 +304,7 @@
                                               nil];
           [qualifier autorelease];
         }
-      [qualifier _gcsAppendToString: sql];
+      [qualifier appendSQLToString: sql];
 
       ex = [channel evaluateExpressionX: sql];
       if (!ex)
@@ -323,7 +337,7 @@
  * @param login the user's login name.
  * @param oldPassword the previous password.
  * @param newPassword the new password.
- * @param perr is not used.
+ * @param perr will be set if the new password is not conform to the policy.
  * @return YES if the password was successfully changed.
  */
 - (BOOL) changePasswordForLogin: (NSString *) login
@@ -331,20 +345,48 @@
                     newPassword: (NSString *) newPassword
                            perr: (SOGoPasswordPolicyError *) perr
 {
+  BOOL didChange, isOldPwdOk, isPolicyOk;
   EOAdaptorChannel *channel;
   GCSChannelManager *cm;
+  NSDictionary *policy;
+  NSEnumerator *policies;
   NSException *ex;
-  NSString *sqlstr;
-  BOOL didChange;
-  BOOL isOldPwdOk;
+  NSRange match;
+  NSString *sqlstr, *regex;
 
+  *perr = -1;
   isOldPwdOk = NO;
+  isPolicyOk = YES;
   didChange = NO;
 
   // Verify current password
   isOldPwdOk = [self checkLogin:login password:oldPassword perr:perr expire:0 grace:0];
 
   if (isOldPwdOk)
+    {
+      if ([_userPasswordPolicy count])
+        {
+          policies = [_userPasswordPolicy objectEnumerator];
+          while (isPolicyOk && (policy = [policies nextObject]))
+            {
+              regex = [policy objectForKey: @"regex"];
+              if (regex)
+                {
+                  match = [newPassword rangeOfString: regex options: NSRegularExpressionSearch];
+                  isPolicyOk = isPolicyOk && match.length > 0;
+                  if (match.length == 0)
+                    {
+                      // [self errorWithFormat: @"Password not conform to policy %@ (%@)", regex, [policy objectForKey: @"label"]];
+                      *perr = PolicyInsufficientPasswordQuality;
+                    }
+                }
+              else
+                [self errorWithFormat: @"Invalid password policy (missing regex): %@", policy];
+            }
+        }
+    }
+
+  if (isOldPwdOk && isPolicyOk)
     {
       // Encrypt new password
       NSString *encryptedPassword = [self _encryptPassword: newPassword];
@@ -440,6 +482,11 @@
   return channel;
 }
 
+- (BOOL) isConnected
+{
+  return [self connection];
+}
+
 - (void) releaseConnection: (id) connection
 {
   GCSChannelManager *cm;
@@ -535,7 +582,7 @@
       qualifier = [[EOOrQualifier alloc] initWithQualifierArray: qualifiers];
       if (domainQualifier)
         qualifier = [[EOAndQualifier alloc] initWithQualifiers: domainQualifier, qualifier, nil];
-      [qualifier _gcsAppendToString: sql];
+      [qualifier appendSQLToString: sql];
 
       ex = [channel evaluateExpressionX: sql];
       if (!ex)
@@ -623,7 +670,7 @@
 
               qualifier = [[EOAndQualifier alloc] initWithQualifiers: q_uid, q_auth, nil];
               [qualifier autorelease];
-              [qualifier _gcsAppendToString: sql];
+              [qualifier appendSQLToString: sql];
 
               ex = [channel evaluateExpressionX: sql];
               if (!ex)
@@ -727,6 +774,30 @@
   return [self _lookupContactEntry: entryID  considerEmail: YES inDomain: domain];
 }
 
+- (void) addVCardProperty: (NSString *) property
+               toCriteria: (NSMutableArray *) criteria
+{
+  static NSDictionary *vCardSQLFieldsTable = nil;
+  NSString *field;
+
+  if (!vCardSQLFieldsTable)
+    vCardSQLFieldsTable = [[NSDictionary alloc] initWithObjectsAndKeys:
+                                                  @"c_cn",            @"fn",
+                                                @"c_cn",              @"n",
+                                                @"mail",              @"email",
+                                                @"mail",              @"mail",
+                                                @"c_telephonenumber", @"tel",
+                                                @"c_o",               @"org",
+                                                @"c_l",               @"adr",
+                                                  nil];
+
+  field = [vCardSQLFieldsTable objectForKey: property];
+  if (field)
+    {
+      [criteria addObjectUniquely: field];
+    }
+}
+
 /* Returns an EOQualifier of the following form:
  * (_domainField = domain OR _domainField = visibleDomain1 [...])
  * Should only be called on SQL sources using _domainField name.
@@ -802,7 +873,7 @@
               if (domainQualifier)
                 {
                   [sql appendString: @" WHERE "];
-                  [domainQualifier _gcsAppendToString: sql];
+                  [domainQualifier appendSQLToString: sql];
                 }
             }
           else
@@ -911,7 +982,7 @@
                   if (domainQualifier)
                     {
                       [sql appendFormat: @" AND ("];
-                      [domainQualifier _gcsAppendToString: sql];
+                      [domainQualifier appendSQLToString: sql];
                       [sql appendFormat: @")"];
                     }
                 }
@@ -948,6 +1019,91 @@
   return results;
 }
 
+
+/**
+ * @see [SOGoContactFolder lookupContactsWithQualifier:andSortOrdering:indDomain:]
+ * @see [SOGoContactSourceFolder lookupContactsWithQualifier:andSortOrdering:indDomain:]
+ * @see [SOGoFolder(CardDAV) _appendComponentsProperties:matchingQualifier:toResponse:context:]
+ */
+- (NSArray *) lookupContactsWithQualifier: (EOQualifier *) qualifier
+                          andSortOrdering: (EOSortOrdering *) ordering
+                                 inDomain: (NSString *) domain
+{
+  static EOAdaptor *adaptor = nil;
+  NSException *ex;
+  NSMutableArray *results;
+  NSMutableString *sql;
+  EOAdaptorChannel *channel;
+  GCSChannelManager *cm;
+
+  results = [NSMutableArray array];
+
+  if (qualifier || !_listRequiresDot)
+    {
+      cm = [GCSChannelManager defaultChannelManager];
+      channel = [cm acquireOpenChannelForURL: _viewURL];
+      if (channel)
+        {
+          if (!adaptor)
+            {
+              EOAdaptorContext *adaptorCtx;
+              adaptorCtx = [channel adaptorContext];
+              adaptor = [adaptorCtx adaptor];
+            }
+          sql = [NSMutableString stringWithFormat: @"SELECT c_name FROM %@ WHERE (", [_viewURL gcsTableName]];
+
+          if (qualifier)
+            [qualifier appendSQLToString: sql
+                             withAdaptor: adaptor];
+          else
+            [sql appendString: @"1 = 1"];
+          [sql appendString: @")"];
+
+          if (_domainField)
+            {
+              if ([domain length])
+                {
+                  EOQualifier *domainQualifier;
+                  domainQualifier = [self visibleDomainsQualifierFromDomain: domain];
+                  if (domainQualifier)
+                    {
+                      [sql appendFormat: @" AND ("];
+                      [domainQualifier appendSQLToString: sql];
+                      [sql appendFormat: @")"];
+                    }
+                }
+              else
+                [sql appendFormat: @" AND %@ IS NULL", _domainField];
+            }
+
+          ex = [channel evaluateExpressionX: sql];
+          if (!ex)
+            {
+              NSDictionary *row;
+              NSMutableDictionary *mutableRow;
+              NSArray *attrs;
+
+              attrs = [channel describeResults: NO];
+
+              while ((row = [channel fetchAttributes: attrs withZone: NULL]))
+                {
+                  mutableRow = [row mutableCopy];
+                  // [mutableRow setObject: self forKey: @"source"];
+                  [results addObject: mutableRow];
+                  [mutableRow release];
+                }
+            }
+          else
+            [self errorWithFormat: @"could not run SQL '%@': %@", sql, ex];
+          [cm releaseChannel: channel];
+        }
+      [self errorWithFormat:@"failed to acquire channel for URL: %@",
+            [_viewURL absoluteString]];
+    }
+
+  return results;
+}
+
 - (void) setSourceID: (NSString *) newSourceID
 {
 }
