@@ -1,6 +1,6 @@
 /*
 
-  Copyright (C) 2006-2015 Inverse inc.
+  Copyright (C) 2006-2021 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -33,8 +33,7 @@
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
-
-#import <Appointments/SOGoAppointmentFolders.h>
+#import <NGExtensions/NSObject+Values.h>
 
 #import <SOGo/NSString+Crypto.h>
 #import <SOGo/NSString+Utilities.h>
@@ -48,6 +47,7 @@
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserManager.h>
+#import <SOGo/SOGoUserSettings.h>
 #import <SOGo/SOGoWebAuthenticator.h>
 
 #if defined(MFA_CONFIG)
@@ -166,13 +166,32 @@
 {
   NSDictionary *jsonError;
 
-  jsonError
-    = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: error]
-                                  forKey: @"LDAPPasswordPolicyError"];
+  jsonError = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: error]
+                                          forKey: @"LDAPPasswordPolicyError"];
   return [self responseWithStatus: 403
             andJSONRepresentation: jsonError];
 }
 
+- (void) _checkAutoReloadWebCalendars: (SOGoUser *) loggedInUser
+{
+  NSDictionary *autoReloadedWebCalendars;
+  NSMutableDictionary *moduleSettings;
+  SOGoUserSettings *us;
+
+  us = [loggedInUser userSettings];
+  moduleSettings = [us objectForKey: @"Calendar"];
+
+  if (moduleSettings)
+    {
+      autoReloadedWebCalendars = [moduleSettings objectForKey: @"AutoReloadedWebCalendars"];
+      if ([[autoReloadedWebCalendars allValues] containsObject: [NSNumber numberWithInt: 1]])
+        {
+          [moduleSettings setObject: [NSNumber numberWithBool: YES] forKey: @"ReloadWebCalendars"];
+          [us synchronize];
+        }
+    }
+}
+
 //
 //
 //
@@ -182,11 +201,11 @@
   WORequest *request;
   WOCookie *authCookie, *xsrfCookie;
   SOGoWebAuthenticator *auth;
-  SOGoAppointmentFolders *calendars;
   SOGoUserDefaults *ud;
+  SOGoUserSettings *us;
   SOGoUser *loggedInUser;
   NSDictionary *params;
-  NSString *username, *password, *language, *domain, *remoteHost, *verificationCode;
+  NSString *username, *password, *language, *domain, *remoteHost;
   NSArray *supportedLanguages, *creds;
 
   SOGoPasswordPolicyError err;
@@ -202,7 +221,6 @@
 
   username = [params objectForKey: @"userName"];
   password = [params objectForKey: @"password"];
-  verificationCode = [params objectForKey: @"verificationCode"];
   language = [params objectForKey: @"language"];
   rememberLogin = [[params objectForKey: @"rememberLogin"] boolValue];
   domain = [params objectForKey: @"domain"];
@@ -218,7 +236,7 @@
       || (expire < 0 && grace > 0)      // password expired, grace still permits login
       || (expire >= 0 && grace == -1))) // password about to expire OR ppolicy activated and passwd never changed
     {
-      NSDictionary *json;
+      NSMutableDictionary *json = [NSMutableDictionary dictionary];
 
       [self logWithFormat: @"successful login from '%@' for user '%@' - expire = %d  grace = %d", remoteHost, username, expire, grace];
 
@@ -231,10 +249,14 @@
         username = [[SOGoUserManager sharedUserManager] getUIDForEmail: username];
 
       loggedInUser = [SOGoUser userWithLogin: username];
+      ud = [loggedInUser userDefaults];
 
 #if defined(MFA_CONFIG)
-      if ([[loggedInUser userDefaults] googleAuthenticatorEnabled])
+      if ([ud totpEnabled])
         {
+          NSString *verificationCode;
+
+          verificationCode = [params objectForKey: @"verificationCode"];
           if ([verificationCode length] == 6 && [verificationCode unsignedIntValue] > 0)
             {
               unsigned int code;
@@ -246,7 +268,7 @@
               const auto time_step = OATH_TOTP_DEFAULT_TIME_STEP_SIZE;
               const auto digits = 6;
 
-              real_secret = [[loggedInUser googleAuthenticatorKey] UTF8String];
+              real_secret = [[loggedInUser totpKey] UTF8String];
 
               auto result = oath_init();
               auto t = time(NULL);
@@ -274,28 +296,46 @@
 
               if (code != [verificationCode unsignedIntValue])
                 {
-                  [self logWithFormat: @"Invalid Google Authenticator key for '%@'", username];
-                  json = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: 1]
-                                                 forKey: @"GoogleAuthenticatorInvalidKey"];
+                  [self logWithFormat: @"Invalid TOTP key for '%@'", username];
+                  [json setObject: [NSNumber numberWithInt: 1]
+                           forKey: @"totpInvalidKey"];
                   return [self responseWithStatus: 403
-                        andJSONRepresentation: json];
+                            andJSONRepresentation: json];
                 }
             } //  if ([verificationCode length] == 6 && [verificationCode unsignedIntValue] > 0)
           else
             {
-              [self logWithFormat: @"Missing Google Authenticator key for '%@', asking it..", username];
-              json = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: 1]
-                                                 forKey: @"GoogleAuthenticatorMissingKey"];
-              return [self responseWithStatus: 202
-                        andJSONRepresentation: json];
+              us = [loggedInUser userSettings];
+              if ([us dictionaryForKey: @"General"] && ![[us dictionaryForKey: @"General"] objectForKey: @"PrivateSalt"])
+                {
+                  // Since v5.3.0, a new salt is used for TOTP. If it's missing, disable TOTP and alert the user.
+                  [ud setTotpEnabled: NO];
+                  [ud synchronize];
+
+                  [self logWithFormat: @"New TOTP key for '%@' must be created", username];
+                  [json setObject: [NSNumber numberWithInt: 1]
+                           forKey: @"totpDisabled"];
+                }
+              else
+                {
+                  [self logWithFormat: @"Missing TOTP key for '%@', asking it..", username];
+                  [json setObject: [NSNumber numberWithInt: 1]
+                           forKey: @"totpMissingKey"];
+                  return [self responseWithStatus: 202
+                            andJSONRepresentation: json];
+                }
             }
         }
 #endif
 
-      json = [NSDictionary dictionaryWithObjectsAndKeys:
-                             [loggedInUser cn], @"cn",
-                                [NSNumber numberWithInt: expire], @"expire",
-                                [NSNumber numberWithInt: grace], @"grace", nil];
+      [self _checkAutoReloadWebCalendars: loggedInUser];
+
+      [json setObject: [loggedInUser cn]
+               forKey: @"cn"];
+      [json setObject: [NSNumber numberWithInt: expire]
+               forKey: @"expire"];
+      [json setObject: [NSNumber numberWithInt: grace]
+               forKey: @"grace"];
 
       response = [self responseWithStatus: 200
                     andJSONRepresentation: json];
@@ -317,14 +357,9 @@
       [context setActiveUser: loggedInUser];
       if (language && [supportedLanguages containsObject: language])
 	{
-	  ud = [loggedInUser userDefaults];
 	  [ud setLanguage: language];
 	  [ud synchronize];
 	}
-
-      calendars = [loggedInUser calendarsFolderInContext: context];
-      if ([calendars respondsToSelector: @selector (reloadWebCalendars:)])
-        [calendars reloadWebCalendars: NO];
     }
   else
     {
@@ -380,7 +415,6 @@
 {
   WOResponse *response;
   NSString *login, *logoutRequest, *newLocation, *oldLocation, *ticket;
-  SOGoAppointmentFolders *calendars;
   SOGoCASSession *casSession;
   SOGoUser *loggedInUser;
   SOGoWebAuthenticator *auth;
@@ -448,9 +482,7 @@
         }
 
       loggedInUser = [SOGoUser userWithLogin: login];
-      calendars = [loggedInUser calendarsFolderInContext: context];
-      if ([calendars respondsToSelector: @selector (reloadWebCalendars:)])
-        [calendars reloadWebCalendars: NO];
+      [self _checkAutoReloadWebCalendars: loggedInUser];
     }
   else
     {
@@ -474,6 +506,7 @@
 {
   WOResponse *response;
   NSString *login, *newLocation, *oldLocation;
+  SOGoUser *loggedInUser;
   WOCookie *saml2LocationCookie;
   WORequest *rq;
 
@@ -498,6 +531,9 @@
           newLocation = [NSString stringWithFormat: @"%@%@",
                                   oldLocation, [login stringByEscapingURL]];
         }
+
+      loggedInUser = [SOGoUser userWithLogin: login];
+      [self _checkAutoReloadWebCalendars: loggedInUser];
     }
   else
     {
@@ -649,62 +685,95 @@
   request = [context request];
   message = [[request contentAsString] objectFromJSONString];
 
-  auth = [[WOApplication application]
-	   authenticatorInContext: context];
-  value = [[context request]
-           cookieValueForKey: [auth cookieNameInContext: context]];
-  creds = [auth parseCredentials: value];
+  auth = [[WOApplication application] authenticatorInContext: context];
+  value = [[context request] cookieValueForKey: [auth cookieNameInContext: context]];
+  creds = nil;
+  username = nil;
 
-  [SOGoSession decodeValue: [SOGoSession valueForSessionKey: [creds objectAtIndex: 1]]
-                  usingKey: [creds objectAtIndex: 0]
-                     login: &username
-                    domain: &domain
-                  password: &password];
+  if (value)
+    {
+      // User is logged in; extract username from session
+      creds = [auth parseCredentials: value];
+
+      [SOGoSession decodeValue: [SOGoSession valueForSessionKey: [creds objectAtIndex: 1]]
+                      usingKey: [creds objectAtIndex: 0]
+                         login: &username
+                        domain: &domain
+                      password: &password];
+    }
+  else
+    {
+      // We are using ppolicy and changing the password upon login
+      username = [message objectForKey: @"userName"];
+      domain = [message objectForKey: @"domain"];
+    }
 
   newPassword = [message objectForKey: @"newPassword"];
   // overwrite the value from the session to compare the actual input
   password = [message objectForKey: @"oldPassword"];
 
-  um = [SOGoUserManager sharedUserManager];
-
-  // This will also update the cached password in memcached.
-  if ([um changePasswordForLogin: username
-                        inDomain: domain
-                     oldPassword: password
-                     newPassword: newPassword
-                            perr: &error])
+  // Validate required parameters
+  if (!username)
     {
-      // We delete the previous session
-      [SOGoSession deleteValueForSessionKey: [creds objectAtIndex: 1]];
-
-      if ([domain isNotNull])
-        {
-          sd = [SOGoSystemDefaults sharedSystemDefaults];
-          if ([sd enableDomainBasedUID] &&
-              [username rangeOfString: @"@"].location == NSNotFound)
-            username = [NSString stringWithFormat: @"%@@%@", username, domain];
-        }
-
-      response = [self responseWith204];
-      authCookie = [auth cookieWithUsername: username
-                                andPassword: newPassword
-                                  inContext: context];
-      [response addCookie: authCookie];
-
-      // We update the XSRF protection cookie
-      creds = [auth parseCredentials: [authCookie value]];
-      xsrfCookie = [WOCookie cookieWithName: @"XSRF-TOKEN"
-                                      value: [[SOGoSession valueForSessionKey: [creds lastObject]] asSHA1String]];
-      [xsrfCookie setPath: [NSString stringWithFormat: @"/%@/", [request applicationName]]];
-      [response addCookie: xsrfCookie];
+     response = [self responseWithStatus: 403
+                              andString: @"Missing 'username' parameter"];
+    }
+  else if (!password)
+    {
+     response = [self responseWithStatus: 403
+                              andString: @"Missing 'oldPassword' parameter"];
+    }
+  else if (!newPassword)
+    {
+     response = [self responseWithStatus: 403
+                              andString: @"Missing 'newPassword' parameter"];
     }
   else
-    response = [self _responseWithLDAPPolicyError: error];
+    {
+      um = [SOGoUserManager sharedUserManager];
+
+      // This will also update the cached password in memcached.
+      if ([um changePasswordForLogin: username
+                            inDomain: domain
+                         oldPassword: password
+                         newPassword: newPassword
+                                perr: &error])
+        {
+          if (creds)
+            {
+              // We delete the previous session
+              [SOGoSession deleteValueForSessionKey: [creds objectAtIndex: 1]];
+            }
+
+          if ([domain isNotNull])
+            {
+              sd = [SOGoSystemDefaults sharedSystemDefaults];
+              if ([sd enableDomainBasedUID] &&
+                  [username rangeOfString: @"@"].location == NSNotFound)
+                username = [NSString stringWithFormat: @"%@@%@", username, domain];
+            }
+
+          response = [self responseWith204];
+          authCookie = [auth cookieWithUsername: username
+                                    andPassword: newPassword
+                                      inContext: context];
+          [response addCookie: authCookie];
+
+          // We update the XSRF protection cookie
+          creds = [auth parseCredentials: [authCookie value]];
+          xsrfCookie = [WOCookie cookieWithName: @"XSRF-TOKEN"
+                                          value: [[SOGoSession valueForSessionKey: [creds lastObject]] asSHA1String]];
+          [xsrfCookie setPath: [NSString stringWithFormat: @"/%@/", [request applicationName]]];
+          [response addCookie: xsrfCookie];
+        }
+      else
+        response = [self _responseWithLDAPPolicyError: error];
+    }
 
   return response;
 }
 
-- (BOOL) isGoogleAuthenticatorEnabled
+- (BOOL) isTotpEnabled
 {
 #if defined(MFA_CONFIG)
   return YES;
