@@ -1,6 +1,6 @@
 /* UIxPreferences.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2021 Inverse inc.
+ * Copyright (C) 2007-2022 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,12 +38,14 @@
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserSettings.h>
 #import <SOGo/SOGoSieveManager.h>
+#import <SOGo/SOGoSource.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUserFolder.h>
 #import <SOGo/SOGoParentFolder.h>
 #import <SOGo/SOGoTextTemplateFile.h>
 #import <SOGo/WOResourceManager+SOGo.h>
 #import <SOGo/SOGoBuild.h>
+#import <SOGo/SOGoPasswordPolicy.h>
 #import <Mailer/SOGoMailAccount.h>
 #import <Mailer/SOGoMailAccounts.h>
 
@@ -160,6 +162,11 @@ static NSArray *reminderValues = nil;
   [super dealloc];
 }
 
+- (NSString *) moduleName
+{
+  return [self commonLabelForKey: @"Preferences"];
+}
+
 - (NSString *) modulePath
 {
   return @"Preferences";
@@ -245,6 +252,29 @@ static NSArray *reminderValues = nil;
   return shortDateFormatText;
 }
 
+//
+// Used by wox template
+//
+- (NSArray *) passwordPolicy
+{
+  NSObject <SOGoSource> *userSource;
+  NSMutableDictionary *translations = [[NSMutableDictionary alloc] init];
+  NSDictionary *policy;
+  NSDictionary *translatedUserPolicy;
+  
+  userSource = [user authenticationSource];
+  
+  for(policy in [userSource userPasswordPolicy]) {
+    [translations setObject:[self labelForKey:[policy objectForKey:@"label"]] 
+                     forKey: [policy objectForKey:@"label"]];
+  }
+  translatedUserPolicy = [SOGoPasswordPolicy createPasswordPolicyLabels: [userSource userPasswordPolicy] 
+                                                                     withTranslations: translations];
+  [translations release];
+  
+  return translatedUserPolicy;
+}
+
 //
 // Used internally
 //
@@ -1444,7 +1474,7 @@ static NSArray *reminderValues = nil;
   if ((v = [o objectForKey: @"defaults"]))
     {
       NSMutableDictionary *sanitizedLabels;
-      NSArray *allKeys, *accounts;
+      NSArray *allKeys, *accounts, *identities;
       NSDictionary *newLabels;
       NSString *name;
       id loginModule;
@@ -1484,67 +1514,81 @@ static NSArray *reminderValues = nil;
           [[[user userDefaults] source] removeObjectForKey: @"SOGoAlternateAvatar"];
         }
 
-      //
-      // We sanitize mail labels
-      //
-      newLabels = [v objectForKey: @"SOGoMailLabelsColors"];
-      if (newLabels && [newLabels isKindOfClass: [NSDictionary class]])
+      if ([self userHasMailAccess])
         {
-          // We encode correctly our keys
-          sanitizedLabels = [NSMutableDictionary dictionary];
-          allKeys = [newLabels allKeys];
-
-          for (i = 0; i < [allKeys count]; i++)
+          //
+          // We sanitize mail labels
+          //
+          newLabels = [v objectForKey: @"SOGoMailLabelsColors"];
+          if (newLabels && [newLabels isKindOfClass: [NSDictionary class]])
             {
-              name = [allKeys objectAtIndex: i];
+              // We encode correctly our keys
+              sanitizedLabels = [NSMutableDictionary dictionary];
+              allKeys = [newLabels allKeys];
 
-              if (![name is7bitSafe])
-                name = [name stringByEncodingImap4FolderName];
+              for (i = 0; i < [allKeys count]; i++)
+                {
+                  name = [allKeys objectAtIndex: i];
 
-              name = [name lowercaseString];
+                  if (![name is7bitSafe])
+                    name = [name stringByEncodingImap4FolderName];
 
-              [sanitizedLabels setObject: [newLabels objectForKey: [allKeys objectAtIndex: i]]
-                                  forKey: name];
+                  name = [name lowercaseString];
+
+                  [sanitizedLabels setObject: [newLabels objectForKey: [allKeys objectAtIndex: i]]
+                                      forKey: name];
+                }
+
+              [v setObject: sanitizedLabels  forKey: @"SOGoMailLabelsColors"];
             }
 
-          [v setObject: sanitizedLabels  forKey: @"SOGoMailLabelsColors"];
-        }
+          //
+          // Keep the primary mail certificate
+          //
+          if ([[[user userDefaults] mailCertificate] length])
+            [v setObject: [[user userDefaults] mailCertificate]  forKey: @"SOGoMailCertificate"];
 
-      //
-      // Keep the primary mail certificate
-      //
-      if ([[[user userDefaults] mailCertificate] length])
-        [v setObject: [[user userDefaults] mailCertificate]  forKey: @"SOGoMailCertificate"];
-
-      //
-      // We sanitize our auxiliary mail accounts
-      //
-      accounts = [v objectForKey: @"AuxiliaryMailAccounts"];
-      if (accounts && [accounts isKindOfClass: [NSArray class]])
-        {
-          if ([accounts count] > 0)
+          //
+          // Keep the original mail identities if none is provided
+          //
+          identities = [v objectForKey: @"SOGoMailIdentities"];
+          if (!identities || ![identities isKindOfClass: [NSArray class]])
             {
-              // The first account is the main system account. The following mapping is required:
-              // - forceDefaultIdentity                => SOGoMailForceDefaultIdentity
-              // - receipts.receiptAction              => SOGoMailReceiptAllow
-              // - receipts.receiptNonRecipientAction  => SOGoMailReceiptNonRecipientAction
-              // - receipts.receiptOutsideDomainAction => SOGoMailReceiptOutsideDomainAction
-              // - receipts.receiptAnyAction           => SOGoMailReceiptAnyAction
-              // - security.alwaysSign                 => SOGoMailCertificateAlwaysSign
-              // - security.alwaysEncrypt              => SOGoMailCertificateAlwaysEncrypt
-              [self _extractMainAccountSettings: [accounts objectAtIndex: 0]  inDictionary: v];
-              if ([self mailAuxiliaryUserAccountsEnabled])
-                accounts = [self _extractAuxiliaryAccounts: accounts];
-              else
-                accounts = [NSArray array];
+              identities = [[user userDefaults] mailIdentities];
+              if ([identities count])
+                [v setObject: identities  forKey: @"SOGoMailIdentities"];
+            }
 
-              [v setObject: accounts  forKey: @"AuxiliaryMailAccounts"];
+          //
+          // We sanitize our auxiliary mail accounts
+          //
+          accounts = [v objectForKey: @"AuxiliaryMailAccounts"];
+          if (accounts && [accounts isKindOfClass: [NSArray class]])
+            {
+              if ([accounts count] > 0)
+                {
+                  // The first account is the main system account. The following mapping is required:
+                  // - forceDefaultIdentity                => SOGoMailForceDefaultIdentity
+                  // - receipts.receiptAction              => SOGoMailReceiptAllow
+                  // - receipts.receiptNonRecipientAction  => SOGoMailReceiptNonRecipientAction
+                  // - receipts.receiptOutsideDomainAction => SOGoMailReceiptOutsideDomainAction
+                  // - receipts.receiptAnyAction           => SOGoMailReceiptAnyAction
+                  // - security.alwaysSign                 => SOGoMailCertificateAlwaysSign
+                  // - security.alwaysEncrypt              => SOGoMailCertificateAlwaysEncrypt
+                  [self _extractMainAccountSettings: [accounts objectAtIndex: 0]  inDictionary: v];
+                  if ([self mailAuxiliaryUserAccountsEnabled])
+                    accounts = [self _extractAuxiliaryAccounts: accounts];
+                  else
+                    accounts = [NSArray array];
+
+                  [v setObject: accounts  forKey: @"AuxiliaryMailAccounts"];
+                }
             }
         }
 
       [[[user userDefaults] source] setValues: v];
 
-      if ([[user userDefaults] synchronize])
+      if ([[user userDefaults] synchronize] && [self userHasMailAccess])
         {
           NSException *error;
           SOGoMailAccount *account;
